

/*************************************************************************/
/****      CONVERT .crd TO .xyz, FIND IMPORTANT ATOMS, COMPUTE Q      ****/
/****      SAVE LOCAL H2O POSITIONS IN FILE WITH THE VALUE OF Q.      ****/ 
/*************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define Nwat 367
#define Nmet 63
#define N (4*Nwat + 5*Nmet)

float angle(float [1+3], float [1+3], float [1+3]);
float dot(float [3+1], float [3+1]);
void normalize(float [1+3]);
void gramschmidt(float [1+3], float [1+3], float [1+3]);
void cross(float [1+3], float [1+3], float [1+3]);
float sqdist(float [1+3], float [1+3]);

float sqdist(float x[1+3], float y[1+3]){
  int k;
  float v[1+3], dp;
  for(k=1;k<=3;k=k+1){
    v[k] = x[k]-y[k];
  }
  dp = dot(v,v);
  return dp;
}

void cross(float R1[1+3], float R2[1+3], float R3[1+3]){
  R3[1]=R1[2]*R2[3]-R1[3]*R2[2];
  R3[2]=0.0-R1[1]*R2[3]+R1[3]*R2[1];
  R3[3]=R1[1]*R2[2]-R1[2]*R2[1];
}

void gramschmidt(float v_in[1+3], float u_in[1+3], float v_out[1+3]){
  float temp, temp2;
  int i;
  temp = dot(v_in, u_in);
  temp2 = dot(u_in, u_in);
  for(i=1;i<=3;i=i+1){
    v_out[i] = v_in[i]-temp*u_in[i]/temp2;
  }
}

void normalize(float x[1+3]){
  float temp;
  int i;
  temp = sqrt(dot(x,x));
  for(i=1;i<=3;i=i+1){
    x[i] = x[i]/temp;
  }
}

float dot(float v[3+1], float u[3+1]){
  int k;
  float dp;
  dp = 0.0;
  for(k=1;k<=3;k=k+1){
    dp = dp+v[k]*u[k];
  }
  return dp;
}

float angle(float A[1+3], float M[1+3], float B[1+3]){
  int k;
  float v1[1+3], v2[1+3];
  for(k=1;k<=3;k=k+1){
    v1[k] = A[k]-M[k];
    v2[k] = B[k]-M[k];
  }
  normalize(v1);
  normalize(v2);
  return (180.0*acos(dot(v1,v2))/3.14159);
}

/*************************************************************************/
/********************************  MAIN  *********************************/
/*************************************************************************/

int main(){
  int i, j, k, iM, c1, kflag, kflag2;
  char indicator[100], currentline[100]={"sometin silly"};
  float xyz[1+N][1+3];   /* the full coordinates    */
  float A[1+3], B[1+3], M[1+3], oldA[1+3], oldB[1+3];
  float Q2, Q1, temp;
  float com[1+3], v[1+3];
  FILE *crd, *autre;

  autre = fopen("i", "w");
  fprintf(autre, "%d", N);
  fclose(autre);
  
  autre = fopen("i", "r");
  fgets(indicator, 100, autre);
  fclose(autre);

  system("colrm 7 20 < crd.crd > i");
  system("colrm 37 < i > xyz.xyz");

  crd = fopen("xyz.xyz", "r");
  while(strstr(currentline,indicator) == NULL){
    fgets(currentline,100,crd);
    /* printf("%s", currentline); */
  }

  for(i=1;i<=N;i=i+1){
    fscanf(crd, "%d ", &j);
    if(j != i){ printf("\n atom corresp: %d != %d", i, j); }
    /* printf("\n %d ", i); */
    for(j=1;j<=3;j=j+1){
      fscanf(crd, "%f ", &temp);
      xyz[i][j] = temp;
      /* printf("%f ", temp); */
    }
  }
  fclose(crd);

  /* dont need the methane H's */

  crd = fopen("xyz.xyz", "w");
  fprintf(crd, "%d\nmandatory", 3*Nwat + Nmet);
  for(i=1;i<=(Nwat*4);i=i+1){
    if(i%4 == 1){
      fprintf(crd, "\nO ");
    }
    if( (i%4==3)  ||  (i%4==0)){
      fprintf(crd, "\nH ");
    }
    if(i%4 != 2){
      fprintf(crd, "%f %f %f", xyz[i][1], xyz[i][2], xyz[i][3]);
    }
  }
  for(i=(Nwat*4+1);i<=N;i=i+5){
    fprintf(crd, "\nC %f %f %f", xyz[i][1], xyz[i][2], xyz[i][3]);
  }
  fclose(crd);

  /***********************************/
  /***  convenient to get M[] now  ***/
  /***********************************/

  i = i-5;
  iM = i;
  for(j=1;j<=3;j=j+1){
    M[j]=xyz[i][j];
  }

  /***********************************/
  /*  local coords from ghost atoms  */
  /***********************************/

  system("./ghost-opt.exe > ghost-opt.txt");

  autre = fopen("ghosts.txt", "r");
  for(i=1;i<=3;i=i+1){
    fscanf(autre, "%f ", &temp);
    oldA[i] = temp;
  }
  for(i=1;i<=3;i=i+1){
    fscanf(autre, "%f ", &temp);
    oldB[i] = temp;
  }
  fclose(autre);
  
  autre = fopen("answer.txt", "r");
  for(i=1;i<=3;i=i+1){
    fscanf(autre, "%f ", &temp);
    A[i] = temp;
  }
  for(i=1;i<=3;i=i+1){
    fscanf(autre, "%f ", &temp);
    B[i] = temp;
  }
  fclose(autre);

  if( (sqdist(A, oldA) < 25.0) && (sqdist(B, oldB) < 25.0) ){
    system("cp answer.txt ghosts.txt");
  }else{
    printf("\n  POSSIBLE ERROR: GHOST ATOM MOVED BY 5A");
  }

  /*****************************/
  /**  ln[d2(A,M)/d2(B,M)]/2  **/
  /*  acos((A-M).(B-M)/||.||)  */
  /*****************************/

  Q1 = log(sqdist(M,A)/sqdist(M,B))/2.0;
  Q2 = angle(A, M, B);
  printf("\nOPs: Q1=%f Q2=%f", Q1, Q2);
  crd = fopen("Q.txt","w");
  fprintf(crd, " %f %f \n", Q1, Q2);
  fclose(crd);

  /****************************/
  /*  local xyz coordinates   */
  /****************************/
  
  for(i=1;i<=3;i=i+1){
    com[i] = (A[i]+B[i])/2.0;
  }
  crd = fopen("localW.xyz", "w");
  fprintf(crd, "\nC  %.3f %.3f %.3f", M[1]-com[1], M[2]-com[2], M[3]-com[3]);
  fprintf(crd, "\nHe %.3f %.3f %.3f", A[1]-com[1], A[2]-com[2], A[3]-com[3]);
  fprintf(crd, "\nHe %.3f %.3f %.3f", B[1]-com[1], B[2]-com[2], B[3]-com[3]);
  k=0;
  kflag = 0;
  for(i=1;i<=(4*Nwat);i=i+4){
    /* are there six waters in -0.5<Q1<0.5  and  0<Q2<100 */
    Q1 = log(sqdist(xyz[i],A)/sqdist(xyz[i],B))/2.0;
    Q2 = angle(A, xyz[i], B);
    if( -0.32 < Q1 ){
      if( Q1 < 0.16 ){
	if( Q2 > 90.0 ){
	  kflag = kflag + 1;
	}
      }
    }       


    if( (sqdist(A,xyz[i]) < 25.0) || (sqdist(B,xyz[i]) < 25.0) || (sqdist(M,xyz[i]) < 25.0) ){
      for(j=1;j<=3;j=j+1){
	v[j] = xyz[i][j]-com[j];
      }
      k = k+1;
      fprintf(crd, "\nO %.3f %.3f %.3f", v[1], v[2], v[3]);
      /* now get the H's for this water too */
      for(j=1;j<=3;j=j+1){
	v[j] = xyz[i+2][j]-com[j];
      }
      fprintf(crd, "\nH %.3f %.3f %.3f", v[1], v[2], v[3]);
      for(j=1;j<=3;j=j+1){
	v[j] = xyz[i+3][j]-com[j];
      }
      fprintf(crd, "\nH %.3f %.3f %.3f", v[1], v[2], v[3]);
    }
  }
  fclose(crd);
  
  i = 3*k + 3;
  system("cp localW.xyz i");
  crd = fopen("j", "w");
  fprintf(crd, "\n%d\nmandatory", i);
  fclose(crd);
  system("cat j i > localW.xyz");

  k=0;
  for(i=(4*Nwat+1);i<=(4*Nwat+5*Nmet);i=i+5){
    if( sqdist(A,xyz[i]) < 16.0 ){
      if( iM != i ){
	k = k+1;
      }
    }
  }
  if(k > 0){
    printf("\n POSSIBLE ERROR: %d UNTAGGED METHANES WITHIN 4A OF GHOST A", k);
  }

  k = 0;
  kflag2 = 0;
  for(i=(4*Nwat+1);i<=(4*Nwat+5*Nmet);i=i+5){
    if( angle(A, xyz[i], B) > 90.0 ){
      if( i != iM ){
	kflag2 = kflag2 + 1;
      }
    }
    if( sqdist(B,xyz[i]) < 16.0 ){
      if( iM != i ){
	k = k+1;
      }
    }
  }
  if(k > 0){
    printf("\n POSSIBLE ERROR: %d UNTAGGED METHANES WITHIN 4A OF GHOST B", k);
  }

  k=0;
  for(i=(4*Nwat+1);i<=(4*Nwat+5*Nmet);i=i+5){
    if( sqdist(M,xyz[i]) < 9.0 ){
      if( iM != i ){
	k = k+1;
      }
    }
  }
  if(k > 0){
    printf("\n POSSIBLE ERROR: %d METHANES WITHIN 3A OF TAGGED METHANE", k);
  }
  // printf("\n");


  /* CONSTRAINT: WATER VACANCY MUST BE BETWEEN ACCEPTOR AND DONOR */
  if( kflag >= 5 ){
    printf("\n ERROR: WATER VACANCY HAS FLED THE SCENE - REJECT");
    Q2 = angle(A, M, B);
    crd = fopen("Q.txt","w");
    fprintf(crd, " %f %f \n", 100.0, Q2);
    fclose(crd);
  }

  /* CONSTRAINT: DOSIE-DO = 2L5L HOPS (SEE PAPER), THIS JOB SAMPLING S5L HOP */
  if( kflag2 > 0 ){
    printf("\n ERROR: DOSIE-DO IN PROGRESS FOR %d METHS - REJECT", kflag2);
    Q2 = angle(A, M, B);
    crd = fopen("Q.txt","w");
    fprintf(crd, " %f %f \n", 100.0, Q2);
    fclose(crd);
  }
  /* SETTING Q=100 IS A REJECTION BECAUSE 100 IS NOT IN ANY OF THE WINDOWS */


  return 0;
}

